﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace WiiSaber
{
    public partial class MainForm : Form
    {
        private SaberSoundController saberSound = null;
        Wii.WiiControllerManager wcm = new Wii.WiiControllerManager();
        TimeSpan updateInterval = TimeSpan.FromMilliseconds(50);
        DateTime lastUpdated = DateTime.MinValue;

        public double LastYAccelValue { get; set; }
        public double LastXAccelValue { get; set; }
        public double LastZAccelValue { get; set; }

        public double LastXAxisChange { get; set; }
        public double LastYAxisChange { get; set; }
        public double LastZAxisChange { get; set; }

        public double LastChange { get; set; }

        public MainForm()
        {
            InitializeComponent();

            saberSound = new SaberSoundController(this);
            this.LastYAccelValue = 0;
            this.LastXAccelValue = 0;
            this.LastZAccelValue = 0;

            this.LastXAxisChange = 0;
            this.LastYAxisChange = 0;
            this.LastZAxisChange = 0;
        }

        private void btnTurnOn_Click(object sender, EventArgs e)
        {
            saberSound.TurnOnLightsaber(this.tbConstantHum.Value);
        }

        private void btnTurnOff_Click(object sender, EventArgs e)
        {
            saberSound.TurnOffLightsaber();
        }

        private void tbSwingIntensity_Scroll(object sender, EventArgs e)
        {
            saberSound.SetSwingIntensity(tbSwingIntensity.Value);
        }

        private void tbConstantHum_Scroll(object sender, EventArgs e)
        {
            saberSound.SetConstantHumVolumePercent(this.tbConstantHum.Value);
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            wcm.Initialize();

            this.lbWiimotes.DataSource = wcm.WiimoteList;

            if (wcm.WiimoteList.Count == 0) { this.gbFindWii.Text = "No Wiimotes Found!"; }
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            wcm.Shutdown();
        }

        private void btnConnectWiimoteToSaber_Click(object sender, EventArgs e)
        {
            Wii.WiiController wc = this.lbWiimotes.SelectedItem as Wii.WiiController;

            if (wc == null)
            {
                MessageBox.Show("Please select a Wii controller from the list.");
                return;
            }

            wc.WiimoteStateChanged += new Wii.WiiController.WiimoteStateChangedEvent(OnWiimoteStateChanged);
            this.lbWiimotes.Enabled = false;
            this.btnConnectWiimoteToSaber.Enabled = false;
            this.tbSwingIntensity.Enabled = false;
        }
        
        void OnWiimoteStateChanged(WiimoteLib.WiimoteState state)
        {
            if (lastUpdated.Add(updateInterval) > DateTime.Now) { return; }

            lastUpdated = DateTime.Now;

            ProcessAxisValues(GetPercentageByCalculation(-1, 1, state.AccelState.Values.X),
                GetPercentageByCalculation(-1, 1, state.AccelState.Values.Y),
                GetPercentageByCalculation(-1, 1, state.AccelState.Values.Z));
        }

        public double GetPercentageByCalculation(double minimum, double maximum, double value)
        {
            if (value > maximum)
            {
                return 100;
            }

            double spread = maximum - minimum;
            if (spread < 0)
            {
                return 0;
            }

            double delta = 0 - minimum;
            double relativeMin = minimum + delta;
            double relativeMax = maximum + delta;
            double relativeVal = value + delta;

            if (relativeVal == 0)
            {
                return 0;
            }

            double percentage = (relativeVal / relativeMax) * 100;

            if (percentage > 100)
            {
                percentage = 100;
            }

            if (percentage < 0)
            {
                percentage = 0;
            }

            return percentage;
        }

        private void ProcessAxisValues(double axisX, double axisY, double axisZ)
        {
            this.LastXAxisChange = System.Math.Abs(axisX - this.LastXAccelValue);
            this.LastYAxisChange = System.Math.Abs(axisY - this.LastYAccelValue);
            this.LastZAxisChange = System.Math.Abs(axisZ - this.LastZAccelValue);

            // measure total motion change from Wii controller, average from last value
            this.LastChange = ((this.LastChange) + (this.LastXAxisChange + this.LastYAxisChange + this.LastZAxisChange)) / 2; //this.GetMaxFromValues(new double[] { this.LastXAxisChange, this.LastYAxisChange, this.LastZAxisChange });

            this.LastXAccelValue = axisX;
            this.LastYAccelValue = axisY;
            this.LastZAccelValue = axisZ;

            try
            {
                this.Invoke(new Action<double>(UpdateSaberForce), this.LastChange);
            }
            catch { }
        }

        private void UpdateSaberForce(double force)
        {
            try
            {
                this.lblSaberForce.Text = force.ToString();
            }
            catch { }
        }

        private double GetMaxFromValues(params double[] values)
        {
            double maxVal = 0;

            foreach (double val in values)
            {
                if (val > maxVal)
                {
                    maxVal = val;
                }
            }

            return maxVal;
        }

        private void tmrUpdateWiimoteParams_Tick(object sender, EventArgs e)
        {
            //this.lblSaberForce.Text = System.Math.Round(this.LastChange, 2).ToString();
        }

        private void tmrUpdateSwingHum_Tick(object sender, EventArgs e)
        {
            UpdateSwingHumFromWiiAccel();
            

        }

        private void UpdateSwingHumFromWiiAccel()
        {
            int targetVolume = this.tbSwingIntensity.Minimum;

            if (this.LastChange > 1)
            {
                targetVolume = Convert.ToInt32(GetPercentageByCalculation(1, 20, this.LastChange));
            }

            if (targetVolume < this.tbSwingIntensity.Minimum)
            {
                targetVolume = this.tbSwingIntensity.Minimum;
            }

            int currentVolume = this.tbSwingIntensity.Value;
            int difference = targetVolume - currentVolume;
            int maxSingleChangeDown = 5;
            int maxSingleChangeUp = 10;

            if (difference > maxSingleChangeUp)
            {
                difference = maxSingleChangeUp;
            }
            else if (difference < 0 - maxSingleChangeDown)
            {
                difference = 0 - maxSingleChangeDown;
            }


            this.tbSwingIntensity.Value = currentVolume + difference;
            saberSound.SetSwingIntensity(currentVolume + difference);
        }
    }
}
